
%%
%% Mandrill results for ICASSP 2009
%% Inpainting region is 30 x 12
%%
%%              Proposed     Criminisi
%% SNR            6.41db        3.06db
%% SSIM           0.65          0.46
%%

% Load test image
imgref = double(rgb2gray(stdimage('mandrill')));

% Set mask for region to be inpainted
rgnmsk = ones(size(imgref)); 
rgnmsk(68:97,174:185) = 0;

% Construct test image with masked region removed
imgtst = imgref;
imgtst(~logical(rgnmsk)) = 0;

% Set parameters and do inpainting
opt.Verbose = 1;
opt.BlockSize = [8 8];
opt.GridStep = [1 4];
opt.NormalBlocks = 1;
opt.NNMaxNum = 128;
opt.NNMaxAngDist = 0.5;
opt.MinKnownFraction = 1.00;
opt.NNSearchRadius = 12;
opt.InitCGTol = 1e-6;
opt.BPDNCGTol = 1e-1;
opt.BPDNLambda = 1e-6;
opt.BPDNMaxIter = 0;
[imgipt, ptm, opt] = inpaintgrey(imgtst, rgnmsk, opt);

% Compute Criminisi result for comparison
[imgcrm,i2,i3,c,d] = cptinpaint(imgtst, rgnmsk == 0);


% Compute SNR
snript = snr(imgref(~rgnmsk), imgipt(~rgnmsk));
snrcrm = snr(imgref(~rgnmsk), imgcrm(~rgnmsk));


% Compute SSIM
ssmpipt = ssimmap(imgref, imgipt);
ssimipt = mean(ssmpipt(~rgnmsk));
ssmpcrm = ssimmap(imgref, imgcrm);
ssimcrm = mean(ssmpcrm(~rgnmsk));


% Display results
disp('             Proposed     Criminisi');
disp(sprintf('SNR           %5.2fdb       %5.2fdb', snript, snrcrm));
disp(sprintf('SSIM          %5.2f         %5.2f', ssimipt, ssimcrm));



if 0,
  figure; imdisp(imgtst); title('Test image');
  figure; imdisp(imgipt); title('Inpainted image');
  figure; imdisp(imgcrm); title('Criminisi inpainted image');
end


if 0,
  [xi,xj] = find(~rgnmsk);
  crd = [min(xi) min(xj) max(xi) max(xj)];
  rgnimg = imbox(imgref, crd);
  imwrite(rgnimg, 'iptdemo6-rgn.png');

  crd = [crd(1)-4 crd(2)-4 crd(3)+4 crd(4)+4];
  
  refrgn = imgref(crd(1):crd(3), crd(2):crd(4));
  imwrite(uint8(refrgn), 'iptdemo6-ref.png');
  
  tstrgn = imgtst(crd(1):crd(3), crd(2):crd(4));
  imwrite(uint8(tstrgn), 'iptdemo6-tst.png');
  
  crmrgn = imgcrm(crd(1):crd(3), crd(2):crd(4));
  imwrite(uint8(crmrgn), 'iptdemo6-crm.png');
  
  seirgn = imgipt(crd(1):crd(3), crd(2):crd(4));
  imwrite(uint8(seirgn), 'iptdemo6-sei.png');  
end

