rsltdsp = 1;
rsltwrt = 0;

% Set parameters
opt = [];
opt.Verbose = 1;
opt.BlockSize = [7 7];
opt.GridStep = [1 3 5];
opt.NormaliseBlocks = 0;
opt.BlkSrcRotations = [0];
opt.NNMaxDictSize = 3*prod(opt.BlockSize);
opt.NNMaxDist = 0.3;
opt.NNMinKnownFrac = 0.75;
opt.IKPBSearchRadius = 30;
opt.IKPBMaxDictSize = 4*3*prod(opt.BlockSize);
opt.InitCGTol = 1e-3;
opt.BPDNLambda = 2e-1;
opt.BPDNOpt.MaxMainIter = 20;
opt.BPDNOpt.CGTol = 1e-4;
opt.BPDNOpt.CGTolScale = 5;

% Load test image
[imgref, rgnmsk, rgncrd] = inpnttst('barbara','r1s3');
imgref = double(imgref);
rgnmsk = double(rgnmsk);

% Set up test image
rgnmskx = repband(rgnmsk, size(imgref,3));
imgtst = imgref;
imgtst(rgnmskx == 0) = 0;

% Do sim. opt. inpainting
[imgjnt, iptinf] = jntoptinpaint(imgtst, rgnmsk, opt);
snript = snr(imgref(~rgnmskx), imgjnt(~rgnmskx));
ssmpipt = ssimmap(imgref, imgjnt);
ssimipt = mean(ssmpipt(~rgnmsk));

% Compute Criminisi result for comparison
[imgcrm, i2, i3, c, d] = cptinpaint(imgtst, rgnmsk == 0);
snrcrm = snr(imgref(~rgnmskx), imgcrm(~rgnmskx));
ssmpcrm = ssimmap(imgref, imgcrm);
ssimcrm = mean(ssmpcrm(~rgnmsk));

% Compare with Wexler result
imgwex = double(imread('fig4/wexler_result.png'));
snrwex = snr(imgref(~rgnmskx), imgwex(~rgnmskx));
ssmpwex = ssimmap(imgref, imgwex);
ssimwex = mean(ssmpwex(~rgnmsk));

 % Display performance measurements
disp('           Joint Opt.     Criminisi        Wexler');
disp(sprintf('SNR           %5.2fdb       %5.2fdb       %5.2fdb', ...
             snript, snrcrm, snrwex));
disp(sprintf('SSIM          %5.2f         %5.2f         %5.2f', ...
             ssimipt, ssimcrm, ssimwex));


if rsltdsp,
  dopt = struct('bregion', rgncrd, 'zgap', 8);
  figure;
  subplot(2,2,1),
  imdisp(imgref, dopt); title('Reference image');
  subplot(2,2,2),
  imdisp(imgjnt, dopt); title('Joint opt. inpainted image');
  subplot(2,2,3),
  imdisp(imgcrm, dopt); title('Criminisi inpainted image');
  subplot(2,2,4),
  imdisp(imgwex, dopt); title('Wexler inpainted image');
end



if rsltwrt,
  zgap = 4;
  zcrd = rgncrd + zgap* [-1 -1 1 1];

  refimg = imbox(imgref, rgncrd);
  refimg = refimg(zcrd(1):zcrd(3),zcrd(2):zcrd(4),:);
  imwrite(uint8(refimg), 'fig4/region_ref.png');

  jntimg = imgjnt(zcrd(1):zcrd(3),zcrd(2):zcrd(4),:);
  imwrite(uint8(jntimg), 'fig4/region_jnt.png');
  
  crmimg = imgcrm(zcrd(1):zcrd(3),zcrd(2):zcrd(4),:);
  imwrite(uint8(crmimg), 'fig4/region_crm.png');
  
  weximg = imgwex(zcrd(1):zcrd(3),zcrd(2):zcrd(4),:);
  imwrite(uint8(weximg), 'fig4/region_wex.png');
end



% >> fig4
% Constructing dictionaries
%   Using 3 grids of 7x7 blocks
%     Grid 1 (offset 1,1) has 28 blocks (5 have NND)
%     Grid 2 (offset 3,3) has 24 blocks (2 have NND)
%     Grid 3 (offset 5,5) has 21 blocks (3 have NND)
%   Vector lengths
%     Concatentated dict. coeff.:  41407
%     Known region penalty      :   3531
%     Block overlap penalty     :   4800
% Constructing initial CG solution  (problem size 8331x41407)
% Performing sparse optimisation (problem size 8331x41407)
% Itn  Fnc       DFid      Reg       WFnc      WDFid     WReg      RelRes    CGIt 
%  Flg
% ------------------------------------------------------------------------------------
%   1  3.70e+01  1.80e+01  9.51e+01  3.69e+01  1.80e+01  9.46e+01  2.54e-04   178   0
%   2  3.31e+01  1.50e+01  9.07e+01  3.31e+01  1.50e+01  9.05e+01  1.04e-04   223   0
%   3  3.12e+01  1.36e+01  8.82e+01  3.12e+01  1.36e+01  8.80e+01  9.47e-05   221   0
%   4  3.01e+01  1.28e+01  8.69e+01  3.01e+01  1.28e+01  8.67e+01  7.43e-05   245   0
%   5  2.94e+01  1.22e+01  8.61e+01  2.94e+01  1.22e+01  8.59e+01  6.38e-05   247   0
%   6  2.90e+01  1.19e+01  8.55e+01  2.89e+01  1.19e+01  8.53e+01  4.48e-05   254   0
%   7  2.86e+01  1.16e+01  8.51e+01  2.86e+01  1.16e+01  8.49e+01  4.83e-05   230   0
%   8  2.84e+01  1.14e+01  8.48e+01  2.83e+01  1.14e+01  8.46e+01  4.03e-05   259   0
%   9  2.82e+01  1.13e+01  8.45e+01  2.81e+01  1.13e+01  8.43e+01  3.77e-05   232   0
%  10  2.80e+01  1.12e+01  8.43e+01  2.80e+01  1.12e+01  8.41e+01  3.30e-05   244   0
%  11  2.79e+01  1.11e+01  8.41e+01  2.79e+01  1.11e+01  8.40e+01  3.48e-05   245   0
%  12  2.78e+01  1.10e+01  8.40e+01  2.78e+01  1.10e+01  8.38e+01  2.96e-05   237   0
%  13  2.77e+01  1.10e+01  8.38e+01  2.77e+01  1.10e+01  8.37e+01  2.70e-05   250   0
%  14  2.77e+01  1.09e+01  8.37e+01  2.76e+01  1.09e+01  8.36e+01  2.41e-05   238   0
%  15  2.76e+01  1.09e+01  8.36e+01  2.76e+01  1.09e+01  8.35e+01  2.55e-05   239   0
%  16  2.75e+01  1.08e+01  8.35e+01  2.75e+01  1.08e+01  8.34e+01  2.57e-05   243   0
%  17  2.75e+01  1.08e+01  8.34e+01  2.75e+01  1.08e+01  8.33e+01  2.87e-05   214   0
%  18  2.75e+01  1.08e+01  8.34e+01  2.74e+01  1.08e+01  8.33e+01  2.23e-05   249   0
%  19  2.74e+01  1.08e+01  8.33e+01  2.74e+01  1.08e+01  8.32e+01  3.17e-05   179   0
%  20  2.74e+01  1.07e+01  8.33e+01  2.74e+01  1.07e+01  8.31e+01  3.14e-05   127   0
% ------------------------------------------------------------------------------------
%              Sim. Opt.    Criminisi        Wexler
% SNR            7.16db        0.70db        5.26db
% SSIM           0.81          0.47          0.74
