rsltdsp = 1;
rsltwrt = 0;

% Set parameters
opt = [];
opt.Verbose = 1;
opt.BlockSize = [9 9];
opt.GridStep = [1 3 5 7];
opt.NormaliseBlocks = 0;
opt.BlkSrcRotations = [-10 -5 0 5 10];
opt.NNMaxDictSize = 3*3*prod(opt.BlockSize);
opt.NNMaxDist = 0.3;
opt.NNMinKnownFrac = 0.6;
opt.IKPBSearchRadius = 50;
opt.IKPBMaxDictSize = 8*3*prod(opt.BlockSize);
opt.DictPruneTol = 2e-5;
opt.InitCGTol = 1e-2;
opt.BPDNLambda = 2e-2;
opt.NNDictSize = [243 486 0];
opt.IKPBMaxKnownFrac = 0.6;
opt.MaxDictSize = 2000;
opt.BPDNOpt.CGTol = 1e-2;
opt.BPDNOpt.CGTolScale = 5;
opt.BPDNOpt.MaxCGIter = 3000;
opt.BPDNOpt.MaxMainIter = 20;

% Load test image
[imgref, rgnmsk, rgncrd] = inpnttst('barbara','r2s2');
imgref = double(imgref);
rgnmsk = double(rgnmsk);

% Set up test image
rgnmskx = repband(rgnmsk, size(imgref,3));
imgtst = imgref;
imgtst(rgnmskx == 0) = 0;

% Do sim. opt. inpainting
[imgjnt, iptinf] = jntoptinpaint(imgtst, rgnmsk, opt);
snript = snr(imgref(~rgnmskx), imgjnt(~rgnmskx));
ssmpipt = ssimmap(imgref, imgjnt);
ssimipt = mean(ssmpipt(~rgnmsk));

% Compute Criminisi result for comparison
[imgcrm, i2, i3, c, d] = cptinpaint(imgtst, rgnmsk == 0);
snrcrm = snr(imgref(~rgnmskx), imgcrm(~rgnmskx));
ssmpcrm = ssimmap(imgref, imgcrm);
ssimcrm = mean(ssmpcrm(~rgnmsk));

% Compare with Wexler result
imgwex = double(imread('fig5/wexler_result.png'));
snrwex = snr(imgref(~rgnmskx), imgwex(~rgnmskx));
ssmpwex = ssimmap(imgref, imgwex);
ssimwex = mean(ssmpwex(~rgnmsk));

 % Display performance measurements
disp('           Joint Opt.     Criminisi        Wexler');
disp(sprintf('SNR           %5.2fdb       %5.2fdb       %5.2fdb', ...
             snript, snrcrm, snrwex));
disp(sprintf('SSIM          %5.2f         %5.2f         %5.2f', ...
             ssimipt, ssimcrm, ssimwex));


if rsltdsp,
  dopt = struct('bregion', rgncrd, 'zgap', 8);
  figure;
  subplot(2,2,1),
  imdisp(imgref, dopt); title('Reference image');
  subplot(2,2,2),
  imdisp(imgjnt, dopt); title('Joint opt. inpainted image');
  subplot(2,2,3),
  imdisp(imgcrm, dopt); title('Criminisi inpainted image');
  subplot(2,2,4),
  imdisp(imgwex, dopt); title('Wexler inpainted image');
end



if rsltwrt,
  zgap = 4;
  zcrd = rgncrd + zgap* [-1 -1 1 1];

  refimg = imbox(imgref, rgncrd);
  refimg = refimg(zcrd(1):zcrd(3),zcrd(2):zcrd(4),:);
  imwrite(uint8(refimg), 'fig5/region_ref.png');

  jntimg = imgjnt(zcrd(1):zcrd(3),zcrd(2):zcrd(4),:);
  imwrite(uint8(jntimg), 'fig5/region_jnt.png');
  
  crmimg = imgcrm(zcrd(1):zcrd(3),zcrd(2):zcrd(4),:);
  imwrite(uint8(crmimg), 'fig5/region_crm.png');
  
  weximg = imgwex(zcrd(1):zcrd(3),zcrd(2):zcrd(4),:);
  imwrite(uint8(weximg), 'fig5/region_wex.png');
end


% >> fig5
% Constructing dictionaries
%   Using 4 grids of 9x9 blocks
%     Grid 1 (offset 1,1) has 12 blocks (5 have NND)
%     Grid 2 (offset 3,3) has 8 blocks (2 have NND)
%     Grid 3 (offset 5,5) has 15 blocks (9 have NND)
%     Grid 4 (offset 7,7) has 12 blocks (6 have NND)
%   Vector lengths
%     Concatentated dict. coeff.:  56469
%     Known region penalty      :   5661
%     Block overlap penalty     :   4320
% Constructing initial CG solution  (problem size 9981x56469)
% Performing sparse optimisation (problem size 9981x56469)
% Itn  Fnc       DFid      Reg       WFnc      WDFid     WReg      RelRes    CGIt 
%  Flg
% ------------------------------------------------------------------------------------
%   1  4.18e+01  3.95e+01  1.12e+02  4.15e+01  3.95e+01  1.01e+02  1.69e-03   124   0
%   2  1.63e+01  1.29e+01  1.73e+02  1.63e+01  1.29e+01  1.72e+02  2.82e-04   491   0
%   3  1.19e+01  8.30e+00  1.81e+02  1.19e+01  8.30e+00  1.81e+02  5.78e-05   947   0
%   4  1.04e+01  6.68e+00  1.84e+02  1.04e+01  6.68e+00  1.84e+02  1.49e-05  1583   0
%   5  9.67e+00  5.96e+00  1.86e+02  9.67e+00  5.96e+00  1.86e+02  9.80e-06  2059   0
%   6  9.32e+00  5.58e+00  1.87e+02  9.32e+00  5.58e+00  1.87e+02  8.27e-06  2093   0
%   7  9.12e+00  5.36e+00  1.88e+02  9.11e+00  5.36e+00  1.88e+02  6.22e-06  1778   0
%   8  8.99e+00  5.23e+00  1.88e+02  8.99e+00  5.23e+00  1.88e+02  5.62e-06  1800   0
%   9  8.90e+00  5.14e+00  1.88e+02  8.90e+00  5.14e+00  1.88e+02  5.43e-06  1544   0
%  10  8.84e+00  5.07e+00  1.89e+02  8.84e+00  5.07e+00  1.89e+02  4.16e-06  1723   0
%  11  8.79e+00  5.02e+00  1.89e+02  8.79e+00  5.02e+00  1.89e+02  4.39e-06  1673   0
%  12  8.76e+00  4.98e+00  1.89e+02  8.75e+00  4.98e+00  1.89e+02  4.02e-06  1681   0
%  13  8.73e+00  4.95e+00  1.89e+02  8.73e+00  4.95e+00  1.89e+02  3.55e-06  1731   0
%  14  8.70e+00  4.92e+00  1.89e+02  8.70e+00  4.92e+00  1.89e+02  3.14e-06  1759   0
%  15  8.69e+00  4.90e+00  1.89e+02  8.69e+00  4.90e+00  1.89e+02  3.00e-06  1605   0
%  16  8.67e+00  4.88e+00  1.89e+02  8.67e+00  4.88e+00  1.89e+02  2.78e-06  1656   0
%  17  8.66e+00  4.87e+00  1.90e+02  8.66e+00  4.87e+00  1.90e+02  2.35e-06  1755   0
%  18  8.65e+00  4.86e+00  1.90e+02  8.65e+00  4.86e+00  1.90e+02  2.39e-06  1626   0
%  19  8.64e+00  4.85e+00  1.90e+02  8.64e+00  4.85e+00  1.90e+02  2.33e-06  1654   0
%  20  8.63e+00  4.84e+00  1.90e+02  8.63e+00  4.84e+00  1.90e+02  2.10e-06  1650   0
% ------------------------------------------------------------------------------------
%              Sim. Opt.    Criminisi        Wexler
% SNR            5.98db        0.28db        2.55db
% SSIM           0.71          0.36          0.54
