rsltdsp = 1;
rsltwrt = 0;

% Set parameters
opt = [];
opt.Verbose = 1;
opt.BlockSize = [9 9];
opt.GridStep = [1 3 5 7];
opt.NormaliseBlocks = 0;
opt.BlkSrcRotations = [0];
opt.NNMaxDictSize = 3*prod(opt.BlockSize);
opt.NNMaxDist = 0.4;
opt.NNMinKnownFrac = 0.75;
opt.IKPBSearchRadius = 30;
opt.IKPBMaxDictSize = 4*3*prod(opt.BlockSize);
opt.InitCGTol = 1e-3;
opt.BPDNLambda = 1e-3;
opt.BPDNOpt.MaxMainIter = 20;
opt.BPDNOpt.CGTol = 1e-4;
opt.BPDNOpt.CGTolScale = 5;
opt.BPDNLambda = 5e-3;

% Load test image
[imgref, rgnmsk, rgncrd] = inpnttst('kodim23','r1s3');
imgref = double(imgref);
rgnmsk = double(rgnmsk);

% Set up test image
rgnmskx = repband(rgnmsk, size(imgref,3));
imgtst = imgref;
imgtst(rgnmskx == 0) = 0;

% Do sim. opt. inpainting
[imgjnt, iptinf] = jntoptinpaint(imgtst, rgnmsk, opt);
snript = snr(imgref(~rgnmskx), imgjnt(~rgnmskx));
ssmpipt = ssimmap(imgref, imgjnt);
ssimipt = mean(ssmpipt(~rgnmsk));

% Compute Criminisi result for comparison
[imgcrm, i2, i3, c, d] = cptinpaint(imgtst, rgnmsk == 0);
snrcrm = snr(imgref(~rgnmskx), imgcrm(~rgnmskx));
ssmpcrm = ssimmap(imgref, imgcrm);
ssimcrm = mean(ssmpcrm(~rgnmsk));

% Compare with Wexler result
imgwex = double(imread('fig7/wexler_result.png'));
snrwex = snr(imgref(~rgnmskx), imgwex(~rgnmskx));
ssmpwex = ssimmap(imgref, imgwex);
ssimwex = mean(ssmpwex(~rgnmsk));

 % Display performance measurements
disp('            Joint Opt.    Criminisi        Wexler');
disp(sprintf('SNR           %5.2fdb       %5.2fdb       %5.2fdb', ...
             snript, snrcrm, snrwex));
disp(sprintf('SSIM          %5.2f         %5.2f         %5.2f', ...
             ssimipt, ssimcrm, ssimwex));


if rsltdsp,
  dopt = struct('bregion', rgncrd, 'zgap', 8);
  figure;
  subplot(2,2,1),
  imdisp(imgref, dopt); title('Reference image');
  subplot(2,2,2),
  imdisp(imgjnt, dopt); title('Joint opt. inpainted image');
  subplot(2,2,3),
  imdisp(imgcrm, dopt); title('Criminisi inpainted image');
  subplot(2,2,4),
  imdisp(imgwex, dopt); title('Wexler inpainted image');
end



if rsltwrt,
  zcrd = [358 262 395 325];
    
  refimg = imbox(imgref, zcrd);
  imwrite(uint8(refimg), 'fig7/whole_ref.png');

  tstimg = imbox(imgtst, zcrd);
  imwrite(uint8(tstimg), 'fig7/whole_tst.png');
  
  jntimg = imgjnt;
  imwrite(uint8(jntimg), 'fig7/whole_jnt.png');
  
  crmimg = imgcrm;
  imwrite(uint8(crmimg), 'fig7/whole_crm.png');
  
  weximg = imgwex;
  imwrite(uint8(weximg), 'fig7/whole_wex.png');

  refimg = imgref(zcrd(1):zcrd(3),zcrd(2):zcrd(4),:);
  imwrite(uint8(refimg), 'fig7/region_ref.png');

  jntimg = imgjnt(zcrd(1):zcrd(3),zcrd(2):zcrd(4),:);
  imwrite(uint8(jntimg), 'fig7/region_jnt.png');
  
  crmimg = imgcrm(zcrd(1):zcrd(3),zcrd(2):zcrd(4),:);
  imwrite(uint8(crmimg), 'fig7/region_crm.png');
  
  weximg = imgwex(zcrd(1):zcrd(3),zcrd(2):zcrd(4),:);
  imwrite(uint8(weximg), 'fig7/region_wex.png');
end


% >> fig7
% Constructing dictionaries
%   Using 4 grids of 9x9 blocks
%     Grid 1 (offset 1,1) has 355 blocks (82 have NND)
%     Grid 2 (offset 3,3) has 348 blocks (73 have NND)
%     Grid 3 (offset 5,5) has 353 blocks (75 have NND)
%     Grid 4 (offset 7,7) has 371 blocks (102 have NND)
%   Vector lengths
%     Concatentated dict. coeff.: 1205414
%     Known region penalty      : 138465
%     Block overlap penalty     : 156222
% Constructing initial CG solution  (problem size 294687x1205414)
% Performing sparse optimisation (problem size 294687x1205414)
% Itn  Fnc       DFid      Reg       WFnc      WDFid     WReg      RelRes    CGIt  Flg
% ------------------------------------------------------------------------------------
%   1  1.62e+01  7.72e+00  1.70e+03  1.62e+01  7.72e+00  1.70e+03  1.85e-04   849   0
%   2  1.47e+01  6.30e+00  1.67e+03  1.47e+01  6.30e+00  1.67e+03  5.18e-05  1614   0
%   3  1.39e+01  5.65e+00  1.66e+03  1.39e+01  5.65e+00  1.65e+03  4.15e-05  1743   0
%   4  1.35e+01  5.28e+00  1.65e+03  1.35e+01  5.28e+00  1.65e+03  3.04e-05  1858   0
%   5  1.32e+01  5.03e+00  1.64e+03  1.32e+01  5.03e+00  1.64e+03  2.85e-05  1801   0
%   6  1.30e+01  4.86e+00  1.64e+03  1.30e+01  4.86e+00  1.63e+03  2.45e-05  1791   0
%   7  1.29e+01  4.74e+00  1.63e+03  1.29e+01  4.74e+00  1.63e+03  2.21e-05  1651   0
%   8  1.28e+01  4.64e+00  1.63e+03  1.28e+01  4.64e+00  1.63e+03  1.99e-05  1687   0
%   9  1.27e+01  4.56e+00  1.63e+03  1.27e+01  4.56e+00  1.63e+03  1.88e-05  1619   0
%  10  1.26e+01  4.50e+00  1.62e+03  1.26e+01  4.50e+00  1.62e+03  1.75e-05  1551   0
%  11  1.26e+01  4.45e+00  1.62e+03  1.26e+01  4.45e+00  1.62e+03  1.58e-05  1509   0
%  12  1.25e+01  4.41e+00  1.62e+03  1.25e+01  4.41e+00  1.62e+03  1.45e-05  1516   0
%  13  1.25e+01  4.37e+00  1.62e+03  1.25e+01  4.37e+00  1.62e+03  1.44e-05  1491   0
%  14  1.24e+01  4.34e+00  1.62e+03  1.24e+01  4.34e+00  1.62e+03  1.33e-05  1460   0
%  15  1.24e+01  4.31e+00  1.62e+03  1.24e+01  4.31e+00  1.62e+03  1.31e-05  1391   0
%  16  1.24e+01  4.28e+00  1.62e+03  1.24e+01  4.28e+00  1.62e+03  1.25e-05  1413   0
%  17  1.23e+01  4.26e+00  1.62e+03  1.23e+01  4.26e+00  1.61e+03  1.21e-05  1434   0
%  18  1.23e+01  4.24e+00  1.61e+03  1.23e+01  4.24e+00  1.61e+03  1.14e-05  1346   0
%  19  1.23e+01  4.22e+00  1.61e+03  1.23e+01  4.22e+00  1.61e+03  1.09e-05  1360   0
%  20  1.23e+01  4.21e+00  1.61e+03  1.23e+01  4.21e+00  1.61e+03  9.84e-06  1368   0
% ------------------------------------------------------------------------------------
%              Sim. Opt.    Criminisi        Wexler
% SNR           16.28db       11.12db       13.39db
% SSIM           0.87          0.76          0.84

