#! /bin/sh

WGET=`which wget`
if [ "$WGET" = '' -o ! -x "$WGET" ]; then
  echo "This script requires the wget command"
  exit 1
fi 

cd extrnl

#
# Get SSIM Index code by Zhou Wang
#
mkdir -p SSIM
cd SSIM
if [ ! -f ssim_index.m ]; then
    wget http://www.ece.uwaterloo.ca/~z70wang/research/ssim/ssim_index.m
fi
cd ..


#
# Get Criminisi, Perez, and Toyama inpainting method code by Sooraj Bhat
#
mkdir -p CPTInpaint
cd CPTInpaint
if [ ! -f cptinpaint.m ]; then
    if [ ! -f inpainting.zip ]; then
	wget http://www.cc.gatech.edu/~sooraj/inpainting/inpainting.zip
    fi
    unzip inpainting.zip
    cd inpainting
    wget http://www.cc.gatech.edu/~sooraj/inpainting/inpaint7.m -O - | tr -d \\r > inpaint7.m
    patch < ../inpaint7.patch
    mv inpaint7.m cptinpaint.m
    mex bestexemplarhelper.c
    cp cptinpaint.m bestexemplarhelper.* ..
    cd ..
    rm -rf inpainting
fi
cd ..


#
# Get Mean Shift Clustering code from Matlab Central (see
# http://www.mathworks.com/matlabcentral/fileexchange/10161)
#
mkdir -p MeanShift
cd MeanShift
if [ ! -f MeanShiftCluster.m ]; then
  if [ ! -f MeanShift.zip ]; then
    wget http://www.mathworks.com/matlabcentral/fx_files/10161/1/MeanShift.zip
  fi
  mkdir tmp
  cp MeanShift.zip tmp
  cd tmp
  unzip MeanShift.zip
  patch < ../MeanShiftCluster.patch
  cp MeanShiftCluster.m ..
  cd ..
  rm -rf tmp
fi
cd ..


#
# Get l1_ls interior-point code from Stanford
#
mkdir -p L1LS
cd L1LS
if [ ! -f l1_ls.m ]; then
  wget http://www.stanford.edu/~boyd/l1_ls/l1_ls_matlab.tar.gz
  tar -zxf l1_ls_matlab.tar.gz
  mv l1_ls_matlab/* .
  rmdir l1_ls_matlab
fi
cd ..


#
# Get SPGL1 sparse reconstruction code from UBC
#
mkdir -p SPGL1
cd SPGL1
if [ ! -f spgl1.m ]; then
  spgl1=spgl1-1.7
  wget -O $spgl1.tgz http://www.cs.ubc.ca/~mpf/downloads/$spgl1.tgz
  tar -zxf $spgl1.tgz
  (cd $spgl1/private && mex oneProjectorMex.c oneProjectorCore.c heap.c \
     -output oneProjectorMex -DNDEBUG)
  mv $spgl1/* .
  rmdir $spgl1
fi
cd ..


#
# Get OMP-Box from Technion
#
mkdir -p OMPBox
cd OMPBox
if [ ! -f omp.m ]; then
  wget http://www.cs.technion.ac.il/~ronrubin/Software/ompbox10.zip
  unzip ompbox10.zip 
  cd private
  echo make > tmpcmd.m
  matlab -nosplash -nodesktop < tmpcmd.m
  rm tmpcmd.m
  cd ..
fi
cd ..


#
# Get KSVD-Box from Technion
#
mkdir -p KSVDBox
cd KSVDBox
if [ ! -f ksvd.m ]; then
  wget http://www.cs.technion.ac.il/~ronrubin/Software/ksvdbox13.zip
  unzip ksvdbox13.zip
  cd private
  echo make > tmpcmd.m
  matlab -nosplash -nodesktop < tmpcmd.m
  rm tmpcmd.m
  cd ..
fi
cd ..


#
# Get ANN Matlab wrapper from Weizmann
#
mkdir -p ANN
cd ANN
if [ ! -f ann_class_compile.m ]; then
  wget http://www.wisdom.weizmann.ac.il/~bagon/matlab_code/ann_wrapper_July2010.tar.gz
  tar -zxvf ann_wrapper_July2010.tar.gz 
  mv ann_wrapper/* .
  rmdir ann_wrapper
  patch -p0 < ann.patch
  echo ann_class_compile > tmpcmd.m
  matlab -nosplash -nodesktop < tmpcmd.m
  rm tmpcmd.m
fi
cd ..
