function C = prunedict(B, t)
  
% prunedict -- Remove equal (or very close) columns from a
%              dictionary matrix
%
% Usage:
%       C = prunedict(B, t)
%
% Input:
%       B           Input dictionary
%       t           Normalised inner product threshold is 1-t
%
% Output:
%       C           Pruned dictionary

if nargin < 2,
  t = 1e-7;
end

[N,M] = size(B);
Bn = normdict(B);
K = 1 - Bn'*Bn;
  
k = 1;
lr = 2:M;
s = ones(1,M);
while k < M && ~isempty(lr),
  s(lr(K(k,lr) < t)) = 0;
  lr = find(s((k+1):end))+k;
  if length(lr) > 1,
    k = lr(1);
    lr = lr(2:end);
  else
    lr = [];
  end
end

C = B(:,s == 1);
  
return
