function DG = dictgrids(img, GRD, BS, nnfn)
  
% dictgrids -- Construct a struct representing a set of
%              dictionaries, one for each image block in a grid set
%
% Usage:
%       DG = dictgrids(img, GRD, BS, nnfn)
%
% Input:
%       img         Image for which dictionaries are required
%       GRD         Grid cell array (see blockgrids function)
%       BS          Block set struct (see blocksetinit function)
%       nnfn        Function handle for nearest neighbour function
%
% Output:
%       DG          A struct representing a set of dictionaries,
%                   one for each image block  

Nbr = GRD.Nbr;
Nbc = GRD.Nbc;

Dict = cell(size(GRD.Grids,1),1);
Lngt = cell(size(GRD.Grids,1),1);
Dist = cell(size(GRD.Grids,1),1);
Indx = cell(size(GRD.Grids,1),1);

for g = 1:size(GRD.Grids,1),

  Dict{g} = cell(size(GRD.Grids{g,1}));
  Lngt{g} = zeros(size(GRD.Grids{g,1}));
  Dist{g} = cell(size(GRD.Grids{g,1}));
  Indx{g} = cell(size(GRD.Grids{g,1}));

  for k = 1:size(GRD.Grids{g,1},1),
    for l = 1:size(GRD.Grids{g,1},2),

      r1 = GRD.Grids{g,1}(k,l);
      c1 = GRD.Grids{g,2}(k,l);

      if r1 ~= 0 && c1 ~= 0,
      
	r2 = r1 + Nbr - 1;
	c2 = c1 + Nbc - 1;
      
	u = img(r1:r2,c1:c2);
	[U, D, Idx] = nnfn(BS.BlkMx, u(:));

	Dict{g}{k,l} = U';
	Lngt{g}(k,l) = size(U,1);
	Dist{g}{k,l} = D;
	Indx{g}{k,l} = Idx;

      end
      
    end
  end
end

DG = struct('Nbr', Nbr, 'Nbc', Nbc, 'Dict', {Dict}, 'Lngt', {Lngt}, ...
	    'Dist', {Dist}, 'Indx', {Indx});

return
