function [BS, img, rndims] = attfaces(Nti, Nbr, Nbc, Nsr, Nsc, rndims)

% attfaces -- Set up training and test data from ATT Faces image set
%
% Usage:
%       [BS, img, rndims] = attfaces(Nti, Nbr, Nbc, Nsr, Nsc, rndims)
%
% Input:
%       Nti         Number of images to use for training data
%       Nbr         Rows in block
%       Nbc         Cols in block
%       Nsr         Row block step
%       Nsc         Column block step
%       rndims      Can be input to use same order if images if desired.
%                   Omit argument to create a new random selection
% Output:
%       BS          Training block set
%       img         Test image
%       rndims      Can be used to find original images

if nargin < 6,
  % Set up vector of image indices
  imgslc = [kron((1:40)',ones(10,1)) kron(ones(40,1), (1:10)')];
  % Random selection of image indices
  rndims = imgslc(randperm(size(imgslc,1)),:);
end
if nargin < 5,
  Nsc = 1;
end
if nargin < 4,
  Nsr = 1;
end

% Determine path to ATT Faces image data
p0 = which('aairpath');
K = strfind(p0, '/');
p1 = p0(1:K(end)-1);
imgpth = [p1 '/data/att-faces'];

% Test image is first image
k = 1;
imgflnm = sprintf('%s/s%d/%d.png', imgpth, rndims(k,1), rndims(k,2));
img = double(imread(imgflnm));

% Remove training images of same subject as selected test image
rndims = rndims(rndims(:,1) ~= rndims(k,1),:);

% Build set of blocks from second through Nti+1 random images
BS = blksetinit([Nbr Nbc]);
for k = 1:Nti,
  imgflnm = sprintf('%s/s%d/%d.png', imgpth, rndims(k,1), rndims(k,2));
  imgb = double(imread(imgflnm));
  BS = blksetadd(BS, imgb, [Nsr Nsc]);
end
BS = blksetend(BS); % Truncate empty allocated space in block array

return
