function msk = cmask(Sz, a)

% cmask -- Construct a circular (elliptical if Nr ~= Nc) mask image
%
% Usage:
%       msk = cmask(Nr, Nc, a)
%
% Input:
%       Sz          Image size, either [Width Height] or N for NxN
%       a           Circle/ellipse radius adjustment factor
%       
% Output:
%       msk         Mask image
  
if nargin < 2,
  a = 1;
end
if prod(size(Sz)) == 1,
  Nr = Sz;
  Nc = Sz;
else
  Nr = Sz(1);
  Nc = Sz(2);
end

ry = (1:Nr)-(Nr+1)/2;
rx = (1:Nc)-(Nc+1)/2;
[bx,by] = meshgrid(ry/max(ry(:)), rx/max(rx(:)));
br0 = max(min(abs(bx(:))),min(abs(by(:))));
msk = bx.^2 + by.^2 <= br0.^2 + a;

return
